@@ -25,24 +25,7 @@
 #include "pango-engine.h"
 #include "pango-utils.h"
 
-
-/* Zero Width characters:
- *
- *  200B  ZERO WIDTH SPACE
- *  200C  ZERO WIDTH NON-JOINER
- *  200D  ZERO WIDTH JOINER
- *  200E  LEFT-TO-RIGHT MARK
- *  200F  RIGHT-TO-LEFT MARK
- *  202A  LEFT-TO-RIGHT EMBEDDING
- *  202B  RIGHT-TO-LEFT EMBEDDING
- *  202C  POP DIRECTIONAL FORMATTING
- *  202D  LEFT-TO-RIGHT OVERRIDE
- *  202E  RIGHT-TO-LEFT OVERRIDE
- */
-
-#define ZERO_WIDTH_CHAR(wc)\
-(((wc) >= 0x200B && (wc) <= 0x200F) || ((wc) >= 0x202A && (wc) <= 0x202E))
-
+#include "basic-common.h"
 
 typedef struct _CharRange CharRange;
 typedef struct _Charset Charset;
@@ -52,7 +35,7 @@ typedef struct _CharCachePointer CharCachePointer;
 typedef struct _MaskTable MaskTable;
 
 typedef PangoGlyph (*ConvFunc) (CharCache   *cache,
-				Charset     *charset,
+				GIConv       cd,
 				const gchar *input);
 
 #define MAX_CHARSETS 32
@@ -103,17 +86,26 @@ struct _CharCachePointer
 };
 
 static PangoGlyph conv_8bit (CharCache  *cache,
-			     Charset    *charset,
+			     GIConv      cd,
 			     const char *input);
 static PangoGlyph conv_eucjp (CharCache  *cache,
-			      Charset    *charset,
+			      GIConv      cd,
 			      const char *input);
 static PangoGlyph conv_16bit (CharCache  *cache,
-			      Charset    *charset,
+			      GIConv      cd,
 			      const char *input);
 static PangoGlyph conv_ucs4 (CharCache  *cache,
-			     Charset    *charset,
+			     GIConv      cd,
 			     const char *input);
+static PangoGlyph conv_16bit_MSB_on (CharCache  *cache,
+			      GIConv      cd,
+			      const char *input);
+static PangoGlyph conv_gb18030_1 (CharCache  *cache,
+			      GIConv      cd,
+			      const char *input);
+static PangoGlyph conv_euctw (CharCache  *cache,
+			      GIConv      cd,
+			      const char *input);
 
 #include "tables-big.i"
 
@@ -128,10 +120,13 @@ static PangoEngineRange basic_ranges[] = {
   { 0x10a0, 0x10ff, "*" }, /* Georgian */
   { 0x1200, 0x16ff, "*" }, /* Ethiopic,Cherokee,Canadian,Ogham,Runic */
   { 0x1e00, 0x1fff, "*" },
-  { 0x2000, 0x9fff, "*" },
+  { 0x2000, 0x33ff, "*" },
+  { 0x3400, 0x9fa5, "*" }, /* Unihan */
+  { 0xa000, 0xa4c6, "*" }, /* Yi */
   { 0xac00, 0xd7a3, "kr" },
+  { 0xe000, 0xf7ee, "*" }, /* HKSCS */
   { 0xf900, 0xfa0b, "kr" },
-  { 0xff00, 0xffe3, "*" }
+  { 0xff00, 0xffed, "*" }
 };
 
 static PangoEngineInfo script_engines[] = {
@@ -271,13 +266,32 @@ find_char (CharCache *cache, PangoFont *font, gunichar wc, const char *input)
     {
       PangoGlyph index;
       PangoGlyph glyph;
+      Charset *charset;
 
-      index = (*mask_table->charsets[i]->conv_func) (cache, mask_table->charsets[i], input);
+      charset = mask_table->charsets[i];
+      if (charset)
+	{
+	  GIConv cd = cache->converters[charset->index];
 
-      glyph = PANGO_X_MAKE_GLYPH (mask_table->subfonts[i], index);
+	  if (charset->id && cd == (GIConv)-1)
+	    {
+	      cd = g_iconv_open (charset->id, "UTF-8");
+	      if (cd == (GIConv)-1)
+		{
+		  g_warning ("Could not load converter from %s to UTF-8", charset->id);
+		  mask_table->charsets[i] = NULL;
+		  continue;
+		}
+	      
+	      cache->converters[charset->index] = cd;      
+	    }
+	  
+	  index = (*charset->conv_func) (cache, cd, input);
+	  glyph = PANGO_X_MAKE_GLYPH (mask_table->subfonts[i], index);
 
-      if (pango_x_has_glyph (font, glyph))
-	return glyph;	  
+	  if (pango_x_has_glyph (font, glyph))
+	    return glyph;	  
+	}
     }
 
   return 0;
@@ -299,26 +313,11 @@ set_glyph (PangoFont *font, PangoGlyphString *glyphs, int i, int offset, PangoGl
   glyphs->glyphs[i].geometry.width = logical_rect.width;
 }
 
-static GIConv
-find_converter (CharCache *cache, Charset *charset)
-{
-  GIConv cd = cache->converters[charset->index];
-  if (cd == (GIConv)-1)
-    {
-      cd = g_iconv_open  (charset->id, "UTF-8");
-      g_assert (cd != (GIConv)-1);
-      cache->converters[charset->index] = cd;
-    }
-
-  return cd;
-}
-
 static PangoGlyph
 conv_8bit (CharCache  *cache,
-	   Charset     *charset,
+	   GIConv      cd,
 	   const char *input)
 {
-  GIConv cd;
   char outbuf;
   
   const char *inptr = input;
@@ -328,8 +327,6 @@ conv_8bit (CharCache  *cache,
 
   inbytesleft = g_utf8_next_char (input) - input;
   
-  cd = find_converter (cache, charset);
-
   g_iconv (cd, (char **)&inptr, &inbytesleft, &outptr, &outbytesleft);
 
   return (guchar)outbuf;
@@ -337,10 +334,9 @@ conv_8bit (CharCache  *cache,
 
 static PangoGlyph
 conv_eucjp (CharCache  *cache,
-	    Charset     *charset,
+	    GIConv      cd,
 	    const char *input)
 {
-  GIConv cd;
   char outbuf[4];
 
   const char *inptr = input;
@@ -350,8 +346,6 @@ conv_eucjp (CharCache  *cache,
 
   inbytesleft = g_utf8_next_char (input) - input;
   
-  cd = find_converter (cache, charset);
-
   g_iconv (cd, (char **)&inptr, &inbytesleft, &outptr, &outbytesleft);
 
   if ((guchar)outbuf[0] < 128)
@@ -366,10 +360,9 @@ conv_eucjp (CharCache  *cache,
 
 static PangoGlyph
 conv_16bit (CharCache  *cache,
-	    Charset     *charset,
+	    GIConv      cd,
 	    const char *input)
 {
-  GIConv cd;
   char outbuf[2];
 
   const char *inptr = input;
@@ -379,8 +372,6 @@ conv_16bit (CharCache  *cache,
 
   inbytesleft = g_utf8_next_char (input) - input;
   
-  cd = find_converter (cache, charset);
-
   g_iconv (cd, (char **)&inptr, &inbytesleft, &outptr, &outbytesleft);
 
   if ((guchar)outbuf[0] < 128)
@@ -389,9 +380,79 @@ conv_16bit (CharCache  *cache,
     return ((guchar)outbuf[0] & 0x7f) * 256 + ((guchar)outbuf[1] & 0x7f);
 }
 
+static PangoGlyph
+conv_16bit_MSB_on (CharCache  *cache,
+	    GIConv      cd,
+	    const char *input)
+{
+  char outbuf[2];
+
+  const char *inptr = input;
+  size_t inbytesleft;
+  char *outptr = outbuf;
+  size_t outbytesleft = 2;
+
+  inbytesleft = g_utf8_next_char (input) - input;
+  
+  g_iconv (cd, (char **)&inptr, &inbytesleft, &outptr, &outbytesleft);
+
+  if ((guchar)outbuf[0] < 128)
+    return outbuf[0];
+  else
+    return (guchar)outbuf[0] * 256 + (guchar)outbuf[1];
+}
+
+static PangoGlyph
+conv_gb18030_1 (CharCache  *cache,
+		GIConv      cd,
+		const char *input)
+{
+  char outbuf[4];
+
+  const char *inptr = input;
+  size_t inbytesleft;
+  char *outptr = outbuf;
+  size_t outbytesleft = 4;
+
+
+  inbytesleft = g_utf8_next_char (input) - input;
+  
+  g_iconv (cd, (char **)&inptr, &inbytesleft, &outptr, &outbytesleft);
+
+  if ((guchar)outbuf[0] < 128)
+    return outbuf[0];
+  else
+    return  12600 * ((guchar)outbuf[0] - 0x81) + 1260 * ((guchar)outbuf[1] - 0x30) + 10 * ((guchar)outbuf[2] - 0x81) + ((guchar)outbuf[3] - 0x30);
+}
+
+static PangoGlyph
+conv_euctw (CharCache  *cache,
+	    GIConv      cd,
+	    const char *input)
+{
+  char outbuf[4];
+
+  const char *inptr = input;
+  size_t inbytesleft;
+  char *outptr = outbuf;
+  size_t outbytesleft = 4;
+
+  inbytesleft = g_utf8_next_char (input) - input;
+  
+  g_iconv (cd, (char **)&inptr, &inbytesleft, &outptr, &outbytesleft);
+
+  /* The first two bytes determine which page of CNS to use; we
+   * get this information from tables-big.i, so ignore them
+   */
+  if ((guchar)outbuf[0] < 128)
+    return outbuf[0];
+  else
+    return ((guchar)outbuf[2] & 0x7f) * 256 + ((guchar)outbuf[3] & 0x7f);
+}
+
 static PangoGlyph
 conv_ucs4 (CharCache  *cache,
-	   Charset     *charset,
+	   GIConv      cd,
 	   const char *input)
 {
   return g_utf8_get_char (input);
@@ -537,7 +598,7 @@ basic_engine_shape (PangoFont        *font,
       const char *input;
 
       wc = g_utf8_get_char (p);
-
+      
       input = p;
       if (analysis->level % 2)
 	if (pango_get_mirror_char (wc, &mirrored_ch))
@@ -548,6 +609,14 @@ basic_engine_shape (PangoFont        *font,
 	    input = buf;
 	  }
 
+      if (wc == 0xa0)	/* non-break-space */
+	{
+	  wc = 0x20;
+	  
+	  g_unichar_to_utf8 (wc, buf);
+	  input = buf;
+	}
+		
       if (ZERO_WIDTH_CHAR (wc))
 	{
 	  set_glyph (font, glyphs, i, p - text, 0);
