@@ -66,7 +66,7 @@ free_cache_entry (LOGFONT             *logfont,
  * pango_win32_font_cache_free:
  * @cache: a #PangoWin32FontCache
  * 
- * Free a #PangoWin32FontCache and all associated memory. All fonts loaded
+ * Frees a #PangoWin32FontCache and all associated memory. All fonts loaded
  * through this font cache will be freed along with the cache.
  **/
 void
@@ -91,7 +91,7 @@ logfont_hash (gconstpointer v)
     lfp->lfItalic +
     lfp->lfWeight/10 +
     lfp->lfOrientation +
-    lfp->lfHeight * 10;
+    abs (lfp->lfHeight) * 10;
 }
 
 static gint
@@ -115,7 +115,7 @@ logfont_equal (gconstpointer   v1,
 /**
  * pango_win32_font_cache_new:
  * 
- * Create a font cache.
+ * Creates a font cache.
  * 
  * Return value: The new font cache. This must be freed with
  * pango_win32_font_cache_free().
@@ -156,7 +156,7 @@ cache_entry_unref (PangoWin32FontCache *cache,
  * @cache: a #PangoWin32FontCache
  * @logfont: a pointer to a LOGFONT structure describing the font to load.
  * 
- * Create a #HFONT from a LOGFONT. The
+ * Creates a #HFONT from a LOGFONT. The
  * result may be newly loaded, or it may have been previously
  * stored
  * 
@@ -184,10 +184,37 @@ pango_win32_font_cache_load (PangoWin32FontCache *cache,
     }
   else
     {
-      BOOL aa;
+      BOOL font_smoothing;
       lf = *lfp;
-      SystemParametersInfo (SPI_GETFONTSMOOTHING, 0, &aa, 0);
-      lf.lfQuality = (aa ? ANTIALIASED_QUALITY : DEFAULT_QUALITY);
+      SystemParametersInfo (SPI_GETFONTSMOOTHING, 0, &font_smoothing, 0);
+      /* If on XP or better, try to use ClearType if the global system
+       * settings ask for it.
+       */
+      if (font_smoothing &&
+	  (pango_win32_os_version_info.dwMajorVersion > 5 ||
+	   (pango_win32_os_version_info.dwMajorVersion == 5 &&
+	    pango_win32_os_version_info.dwMinorVersion >= 1)))
+	{
+	  UINT smoothing_type;
+
+#ifndef SPI_GETFONTSMOOTHINGTYPE 
+#define SPI_GETFONTSMOOTHINGTYPE 0x200a
+#endif
+#ifndef FE_FONTSMOOTHINGCLEARTYPE
+#define FE_FONTSMOOTHINGCLEARTYPE 2
+#endif
+#ifndef CLEARTYPE_QUALITY
+#define CLEARTYPE_QUALITY 5
+#endif
+	  SystemParametersInfo (SPI_GETFONTSMOOTHINGTYPE, 0, &smoothing_type, 0);
+	  lf.lfQuality =
+	    (font_smoothing ?
+	     (smoothing_type == FE_FONTSMOOTHINGCLEARTYPE ?
+	      CLEARTYPE_QUALITY : ANTIALIASED_QUALITY) :
+	     DEFAULT_QUALITY);
+	}
+      else
+	lf.lfQuality = (font_smoothing ? ANTIALIASED_QUALITY : DEFAULT_QUALITY);
       lf.lfCharSet = DEFAULT_CHARSET;
       for (tries = 0; ; tries++)
 	{
@@ -327,7 +354,7 @@ pango_win32_font_cache_load (PangoWin32FontCache *cache,
  * @cache: a #PangoWin32FontCache
  * @hfont: the HFONT to unload
  * 
- * Free a font structure previously loaded with pango_win32_font_cache_load()
+ * Frees a font structure previously loaded with pango_win32_font_cache_load().
  **/
 void
 pango_win32_font_cache_unload (PangoWin32FontCache *cache,
