@@ -1,5 +1,5 @@
 /* Pango
- * pango-font.h: Font handling
+ * pangox-fontmap.c: X font handling
  *
  * Copyright (C) 2000 Red Hat Software
  *
@@ -19,7 +19,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -130,6 +129,7 @@ static void     pango_x_font_map_class_init (PangoFontMapClass *class);
 
 static void       pango_x_font_map_finalize      (GObject                      *object);
 static PangoFont *pango_x_font_map_load_font     (PangoFontMap                 *fontmap,
+						  PangoContext                 *context,
 						  const PangoFontDescription   *description);
 static void       pango_x_font_map_list_families (PangoFontMap                 *fontmap,
 						  PangoFontFamily            ***families,
@@ -217,6 +217,16 @@ pango_x_font_map_class_init (PangoFontMapClass *class)
 
 static GList *fontmaps = NULL;
 
+/**
+ * pango_x_font_map_for_display:
+ * @display: an X #Display.
+ *
+ * Returns a #PangoXFontMap for @display. Font maps are cached and should 
+ * not be freed. If the font map for a display is no longer needed, it can 
+ * be released with pango_x_shutdown_display().
+ *
+ * Returns: a #PangoXFontMap for @display.
+ **/
 PangoFontMap *
 pango_x_font_map_for_display (Display *display)
 {
@@ -387,6 +397,7 @@ pango_x_get_font_family (PangoXFontMap *xfontmap,
 
 static PangoFont *
 pango_x_font_map_load_font (PangoFontMap               *fontmap,
+			    PangoContext               *context,
 			    const PangoFontDescription *description)
 {
   PangoXFontMap *xfontmap = (PangoXFontMap *)fontmap;
@@ -971,6 +982,7 @@ pango_x_get_xlfd_field (const char *fontname,
 			char       *buffer)
 {
   const char *t1, *t2;
+  char *p;
   int countdown, len, num_dashes;
   
   if (!fontname)
@@ -1000,7 +1012,8 @@ pango_x_get_xlfd_field (const char *fontname,
       strncpy (buffer, t1, len);
       buffer[len] = 0;
       /* Convert to lower case. */
-      g_strdown (buffer);
+      for (p = buffer; *p; p++)
+	*p = g_ascii_tolower (*p);
     }
   else
     strcpy(buffer, "(nil)");
@@ -1280,6 +1293,14 @@ free_coverages_foreach (gpointer key,
   pango_coverage_unref (value);
 }
 
+/**
+ * pango_x_font_map_get_font_cache:
+ * @font_map: a #PangoXFontMap.
+ *
+ * Obtains the font cache associated with the given font map.
+ *
+ * Returns: the #PangoXFontCache of @font_map.
+ **/
 PangoXFontCache *
 pango_x_font_map_get_font_cache (PangoFontMap *font_map)
 {
@@ -1515,12 +1536,12 @@ pango_x_face_get_coverage (PangoXFace      *xface,
 		  g_hash_table_insert (coverage_hash, map_entry->info->id, coverage);
 		}
 	  
-	  font_level = pango_coverage_get (coverage, ch);
-	  if (font_level == PANGO_COVERAGE_EXACT && !map_entry->is_exact)
-	    font_level = PANGO_COVERAGE_APPROXIMATE;
-
-	  if (font_level != PANGO_COVERAGE_NONE)
-	    pango_coverage_set (result, ch, font_level);
+	      font_level = pango_coverage_get (coverage, ch);
+	      if (font_level == PANGO_COVERAGE_EXACT && !map_entry->is_exact)
+		font_level = PANGO_COVERAGE_APPROXIMATE;
+	      
+	      if (font_level != PANGO_COVERAGE_NONE)
+		pango_coverage_set (result, ch, font_level);
 	    }
 	}
       
